﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using KandaAlpha.Application.Interfaces;
using KandaAlpha.Domain.Repository;
using KandaAlpha.Domain.Model.Entities;
using Rhino.Mocks; 

namespace KandaAlpha.Application.Impl.Tests.Service
{
    /// <summary>
    /// Summary description for CustomerServiceTest
    /// </summary>
    [TestClass]
    public class CustomerServiceTest : BaseTest 
    {
        private ICustomerService _customerService;
        private ICustomerRepository _customerRepository;
        private Customer _fakeCustomer; 
        public CustomerServiceTest()
        {
            BuildMocks();
        }
        
        private void BuildMocks()
        {
            _customerRepository = _mocks.Stub<ICustomerRepository>();
            _fakeCustomer = CreateFake<Customer>(); 

            using (_mocks.Record())
            {
                SetupResult
                    .For(_customerRepository.GetBestCustomer())
                    .Return(_fakeCustomer);

            }

            _customerService = new CustomerService(_customerRepository); 

        }
        

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void GetBestCustomer_Pass()
        {
            var actual = _customerService.GetBestCustomer();

            Assert.IsNotNull(actual);
            Assert.AreEqual(1, actual.ID);
            VerifyFake(actual); 
        }

        [TestMethod]
        public void Add_Pass()
        {
            var customer = _fakeCustomer; 

            _customerService.Add(customer);

            _customerRepository.AssertWasCalled(c => c.Add(customer)); 
        
        }
    }
}
